local PitBull4 = PitBull4
if not PitBull4 then
	error("PitBull4_VengeanceBar requires PitBull4")
end
local PitBull4_LuaTexts = PitBull4:GetModule("LuaTexts", true)

local MaxVengeanceAP
local BaseValue = 0
local PlayerClass = select(2, UnitClass("player"))

local ClassTankTree = nil
if PlayerClass == "WARRIOR" then ClassTankTree = 3
elseif PlayerClass == "DEATHKNIGHT" then ClassTankTree = 1
elseif PlayerClass == "DRUID" then ClassTankTree = 2
elseif PlayerClass == "PALADIN" then ClassTankTree = 2
end

local BEAR_FORM = BEAR_FORM --5

--tooltip scanner to get the attack power value from the vengeance buff
local GetVengeanceAP
do
	local tooltip = CreateFrame("GameTooltip", "VengeanceBarTooltip", nil, "GameTooltipTemplate")
	tooltip:SetOwner(UIParent, "ANCHOR_NONE")

	local vengeanceSpell = GetSpellInfo(93098)

	function GetVengeanceAP()
		if UnitBuff("player", vengeanceSpell) then
			tooltip:ClearLines()
			tooltip:SetUnitBuff("player", vengeanceSpell)
			return tonumber(string.match(VengeanceBarTooltipTextLeft2:GetText() or "", "%d+")) or 0
		end
		return 0
	end
end

local function UpdateBaseValue(_, level)
	local effectiveStat = select(2, UnitStat("player", 3))
	local baseStam = min(20, effectiveStat)
	local increase = (baseStam + ((effectiveStat - baseStam) * UnitHPPerStamina("player"))) * GetUnitMaxHealthModifier("player")
	BaseValue = floor((UnitHealthMax("player") - increase) / 10)
end

--Vengeance caps at a number equal to Stamina plus 10% of base health
local function GetVengeanceAPMax()
	return UnitStat("player", 3) + BaseValue
end

--Allow for getting the AP value in LuaTexts
if PitBull4_LuaTexts then
	PitBull4_LuaTexts.ScriptEnv.VengeanceAP = GetVengeanceAP
	PitBull4_LuaTexts.ScriptEnv.VengeanceAPMax = GetVengeanceAPMax
end
--[[
Lua:Vengeance

if IsMouseOver() then
  local cur,max = VengeanceAP(),VengeanceAPMax()
  return "%d/%d (%d%%)",cur,max,Percent(cur,max)
end
return ConfigMode()
--]]

local L = {}
L["Vengeance bar"] = "Vengeance bar"
L["Show a bar indicating the attack power gained from your Vengeance."] = "Show a bar indicating the attack power gained from your Vengeance."
if GetLocale() == "zhTW" then
	L["Vengeance bar"] = "復仇欄位"
	L["Show a bar indicating the attack power gained from your Vengeance."] = "顯示從你的復仇提升的攻擊強度。"
end

local PitBull4_VengeanceBar = PitBull4:NewModule("VengeanceBar", "AceEvent-3.0")

PitBull4_VengeanceBar:SetModuleType("bar")
PitBull4_VengeanceBar:SetName(L["Vengeance bar"])
PitBull4_VengeanceBar:SetDescription(L["Show a bar indicating the attack power gained from your Vengeance."])
PitBull4_VengeanceBar:SetDefaults({
	size = 1,
	position = 4,
	enabled = false,
})

function PitBull4_VengeanceBar:OnEnable()
	self:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED")
	self:RegisterEvent("PLAYER_TALENT_UPDATE", "ACTIVE_TALENT_GROUP_CHANGED")
	self:RegisterEvent("PLAYER_LEVEL_UP", UpdateBaseValue)

	UpdateBaseValue()
end

function PitBull4_VengeanceBar:GetValue(frame)
	if MaxVengeanceAP and frame.unit == "player" then
		return GetVengeanceAP() / MaxVengeanceAP
	end
end

function PitBull4_VengeanceBar:GetExampleValue()
	return 0.7
end

function PitBull4_VengeanceBar:GetColor()
	local color = PitBull4.PowerColors.POWER_TYPE_HEAT
	if color then
		return color[1], color[2], color[3]
	end
end


function PitBull4_VengeanceBar:ACTIVE_TALENT_GROUP_CHANGED()
	self:UnregisterEvent("UNIT_MAXHEALTH")
	self:UnregisterEvent("UNIT_AURA")
	self:UnregisterEvent("UPDATE_SHAPESHIFT_FORM")

	--check spec for vengeance
	if GetPrimaryTalentTree() == ClassTankTree then
		self:RegisterEvent("UNIT_MAXHEALTH")
		self:RegisterEvent("UNIT_AURA")

		--burr shenanigans
		if PlayerClass == "DRUID" then
			self:RegisterEvent("UPDATE_SHAPESHIFT_FORM")
			self:UPDATE_SHAPESHIFT_FORM()
			return
		end

		MaxVengeanceAP = GetVengeanceAPMax()
	else
		MaxVengeanceAP = nil
	end

	self:UpdateAll()
end

function PitBull4_VengeanceBar:UNIT_AURA(_, unit)
	if unit == "player" and MaxVengeanceAP then
		self:UpdateForUnitID(unit)
	end
end

function PitBull4_VengeanceBar:UNIT_MAXHEALTH(_, unit)
	if unit == "player" and MaxVengeanceAP then
		MaxVengeanceAP = GetVengeanceAPMax()
		self:UpdateForUnitID(unit)
	end
end

function PitBull4_VengeanceBar:UPDATE_SHAPESHIFT_FORM()
	if GetShapeshiftFormID() == BEAR_FORM then
		MaxVengeanceAP = GetVengeanceAPMax()
	else
		MaxVengeanceAP = nil
	end

	self:UpdateForUnitID("player")
end
